
/* Generated data (by glib-mkenums) */

#include "shell-enum-types.h"
/* enumerations from "./shell-app.h" */
#include "./shell-app.h"
GType
shell_app_state_get_type(void) {
  static GType enum_type_id = 0;
  if (G_UNLIKELY (!enum_type_id))
    {
      static const GEnumValue values[] = {
        { SHELL_APP_STATE_STOPPED, "SHELL_APP_STATE_STOPPED", "stopped" },
        { SHELL_APP_STATE_STARTING, "SHELL_APP_STATE_STARTING", "starting" },
        { SHELL_APP_STATE_RUNNING, "SHELL_APP_STATE_RUNNING", "running" },
        { SHELL_APP_STATE_BUSY, "SHELL_APP_STATE_BUSY", "busy" },
        { 0, NULL, NULL }
      };
      enum_type_id = g_enum_register_static("ShellAppState", values);
    }
  return enum_type_id;
}
/* enumerations from "./shell-keybinding-modes.h" */
#include "./shell-keybinding-modes.h"
GType
shell_key_binding_mode_get_type(void) {
  static GType enum_type_id = 0;
  if (G_UNLIKELY (!enum_type_id))
    {
      static const GFlagsValue values[] = {
        { SHELL_KEYBINDING_MODE_NONE, "SHELL_KEYBINDING_MODE_NONE", "none" },
        { SHELL_KEYBINDING_MODE_NORMAL, "SHELL_KEYBINDING_MODE_NORMAL", "normal" },
        { SHELL_KEYBINDING_MODE_OVERVIEW, "SHELL_KEYBINDING_MODE_OVERVIEW", "overview" },
        { SHELL_KEYBINDING_MODE_LOCK_SCREEN, "SHELL_KEYBINDING_MODE_LOCK_SCREEN", "lock-screen" },
        { SHELL_KEYBINDING_MODE_UNLOCK_SCREEN, "SHELL_KEYBINDING_MODE_UNLOCK_SCREEN", "unlock-screen" },
        { SHELL_KEYBINDING_MODE_LOGIN_SCREEN, "SHELL_KEYBINDING_MODE_LOGIN_SCREEN", "login-screen" },
        { SHELL_KEYBINDING_MODE_MESSAGE_TRAY, "SHELL_KEYBINDING_MODE_MESSAGE_TRAY", "message-tray" },
        { SHELL_KEYBINDING_MODE_SYSTEM_MODAL, "SHELL_KEYBINDING_MODE_SYSTEM_MODAL", "system-modal" },
        { SHELL_KEYBINDING_MODE_LOOKING_GLASS, "SHELL_KEYBINDING_MODE_LOOKING_GLASS", "looking-glass" },
        { SHELL_KEYBINDING_MODE_TOPBAR_POPUP, "SHELL_KEYBINDING_MODE_TOPBAR_POPUP", "topbar-popup" },
        { SHELL_KEYBINDING_MODE_ALL, "SHELL_KEYBINDING_MODE_ALL", "all" },
        { 0, NULL, NULL }
      };
      enum_type_id = g_flags_register_static("ShellKeyBindingMode", values);
    }
  return enum_type_id;
}
/* enumerations from "./shell-network-agent.h" */
#include "./shell-network-agent.h"
GType
shell_network_agent_response_get_type(void) {
  static GType enum_type_id = 0;
  if (G_UNLIKELY (!enum_type_id))
    {
      static const GEnumValue values[] = {
        { SHELL_NETWORK_AGENT_CONFIRMED, "SHELL_NETWORK_AGENT_CONFIRMED", "confirmed" },
        { SHELL_NETWORK_AGENT_USER_CANCELED, "SHELL_NETWORK_AGENT_USER_CANCELED", "user-canceled" },
        { SHELL_NETWORK_AGENT_INTERNAL_ERROR, "SHELL_NETWORK_AGENT_INTERNAL_ERROR", "internal-error" },
        { 0, NULL, NULL }
      };
      enum_type_id = g_enum_register_static("ShellNetworkAgentResponse", values);
    }
  return enum_type_id;
}

/* Generated data ends here */

